package net.martinblog.wakemeup;

import java.util.ArrayList;
import java.util.List;
import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.MyLocationOverlay;
import com.google.android.maps.Overlay;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Point;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.view.MotionEvent;
import android.widget.Toast;

public class WakeMeUpActivity extends MapActivity implements LocationListener {
	
	MapView 			maps;
	MapController 		mc;
	
	LocationManager		locManager;
	MyLocationOverlay	myLocation;
	
	List<Overlay>		overlays;
	List<GeoPoint>		list =  new ArrayList<GeoPoint>();
	
	// display image on the map
	class MapOverlay extends com.google.android.maps.Overlay
    {
		GeoPoint gp;
		
		public MapOverlay(GeoPoint gp)
		{
			this.gp = gp;			
		}
		
    	@Override
        public boolean draw(Canvas canvas, MapView mapView, boolean shadow, long when) 
        {
        	super.draw(canvas, mapView, shadow); 
        	
            Point screenPts = new Point();
            maps.getProjection().toPixels(gp, screenPts);
            
            Bitmap bmp = BitmapFactory.decodeResource( getResources(), R.drawable.pin );            
            canvas.drawBitmap(bmp, screenPts.x-32, screenPts.y-32, null);
            
            return true;
        }	
    } 
	
	class TouchOverlay extends com.google.android.maps.Overlay
	{
		float posX = 0;
		float posY = 0;

		GeoPoint gp;

		@Override
        public boolean onTouchEvent(MotionEvent event, MapView mapview)
    	{			
			if(event.getAction() == MotionEvent.ACTION_DOWN)
			{
				posX = event.getX();
				posY = event.getY();
				
//				Toast.makeText(WakeMeUpActivity.this, "" + posX + ", " + posY, Toast.LENGTH_SHORT).show();
			}
			
    		if(event.getAction() == MotionEvent.ACTION_UP)
    		{
    			//   
    			float x = event.getX();
				float y = event.getY();

				// ġ  ̵  Ÿ
				float dist = Math.abs(posX - x) + Math.abs(posY - y);
							
				// ġ ð 
    			float elapse = event.getEventTime() - event.getDownTime();
    			
    			if( elapse > 500.0 && dist < 30.0 )
    			{
    				gp = mapview.getProjection().fromPixels((int)x, (int)y);
    				
//				    Toast.makeText(WakeMeUpActivity.this, "Ŭ!", Toast.LENGTH_SHORT).show();
    				AlertDialog.Builder alert = new AlertDialog.Builder(WakeMeUpActivity.this);
    				alert.setIcon(R.drawable.ic_launcher);
    				alert.setTitle("˶ ");
    				alert.setMessage("̰ ˶ Ͻðڽϱ?");
    				
    				alert.setPositiveButton("", new DialogInterface.OnClickListener() {
    					
    					@Override
    					public void onClick(DialogInterface dialog, int which) {

    						// insert current position to the alarm list
    						list.add(gp);
    						overlays.add( new MapOverlay(gp) );
    					}
    				});
    				
    				alert.setNegativeButton("", new DialogInterface.OnClickListener() {
						
						@Override
						public void onClick(DialogInterface dialog, int which) {
							// TODO Auto-generated method stub
							
						}
					});
    				
					alert.show();
			    }
    		}
    
            return false;
        }
	}
	
	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);

		// create map
		maps = (MapView)findViewById(R.id.map);
		maps.setBuiltInZoomControls(true);
		
		// get overlay
		overlays = maps.getOverlays();
		
        // Set Message
        Toast.makeText(this, "   ˶  ϼ!", Toast.LENGTH_LONG).show();
	}
	
	@Override
	public void onStart() {
		super.onStart();
		
		// get the handle of location listener
        locManager = (LocationManager)getSystemService(Context.LOCATION_SERVICE);
        
        // request GPS location update
        locManager.requestLocationUpdates( LocationManager.GPS_PROVIDER, 0, 0, this );
        
        // request Network location update
        locManager.requestLocationUpdates( LocationManager.NETWORK_PROVIDER, 1000, 0, this );
        
        // get last location 
    	Location lastLocation = locManager.getLastKnownLocation( LocationManager.GPS_PROVIDER );
        
		// initial zoom and position
		mc = maps.getController();
		mc.setZoom(18);

        if( lastLocation != null )
        {
        	GeoPoint gp = new GeoPoint( (int)(lastLocation.getLatitude() * 1E6), (int)(lastLocation.getLongitude() * 1E6) );
        	mc.animateTo(gp);
			mc.setCenter(gp);
        }
        
        // set MyLocationOverlay
        myLocation = new MyLocationOverlay( WakeMeUpActivity.this, maps );
        overlays.add(myLocation);
        overlays.add(new TouchOverlay());
        
        maps.invalidate();
		
	}
	
	@Override
	public void onResume()
	{
		super.onResume();
		
		myLocation.enableMyLocation();
		myLocation.enableCompass();
	}
	
	@Override
	public void onPause()
	{
		super.onPause();
		
		myLocation.disableMyLocation();
		myLocation.disableCompass();
	}
	
	@Override
	public void onStop()
	{
		super.onStop();
		locManager.removeUpdates(this);
	}

	@Override
	protected boolean isRouteDisplayed() {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public void onLocationChanged(Location location) {
		
		// get GPS position
		double latitude = location.getLatitude();
		double longitude = location.getLongitude();
		
		GeoPoint gp = new GeoPoint((int)(latitude*1E6), (int)(longitude*1E6));		
		
		maps.getController().animateTo(gp);
		maps.getController().setCenter(gp);
		        		
		// calculate the distances
		for(int i=0;i<list.size();i++)
		{
			double dist = 0.0;
			
			Location loc = new Location("current");
			loc.setLatitude((float)list.get(i).getLatitudeE6() / 1E6);
			loc.setLongitude((float)list.get(i).getLongitudeE6() / 1E6);
			
			dist = location.distanceTo( loc );
			
//			Toast.makeText(WakeMeUpActivity.this, " " + (i+1) + "  " + (int)dist + "m ҽϴ." , Toast.LENGTH_SHORT).show();
			
			if( dist < 50 /* distance to notify(changeable, meter) */ )	
			{
				// notify
				Toast.makeText(WakeMeUpActivity.this, "˶   50m ̳ ϼ̽ϴ.", Toast.LENGTH_LONG).show();
				
				// delete list item
				list.remove(i);
				overlays.remove(i+2 /* except current position and touch overlay */ );
				
				maps.invalidate();
				
				// end of loop
				break;
			}
		}

	}

	@Override
	public void onProviderDisabled(String provider) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void onProviderEnabled(String provider) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void onStatusChanged(String provider, int status, Bundle extras) {
		// TODO Auto-generated method stub
		
	}
}